#include "ca.hpp" //Include this first, because it includes windows.
#include "carule.hpp" //Include this second
#include <math.h>
/* Usage.

	Define a CA rule using the data names defined in carule.hpp.  It is
handy to keep all of your past rules in this file and simply comment in
exactly one of the #define statements to pick one rule.

	Compile this as a DLL target using the Standard Libraries.  For use with
the 16 bit CAPOW choose the Large target model type, for use with the 32
bit CAPOW choose the GUI target model type.  In the 16 bit case, do not
use a carule.def file. Using no *.def file and accepting the 16-bit
compiler warning, "No module definition file, using defualts" works fine.

	You can change the name of your output *.DLL by editing the target
node name.  Alternately you can rename the *.DLL after it is made.

	  If you are running both 16 bit and 32 bit versions of CAPOW on the
same machine you should add "16" to the names of the 16 bit DLLs and "32" to
the names of the 32 bit *.DLLs to avoid loading a 32 bit DLL in a 16-bit 
CAPOW or vice-versa, as this will cause a crash.

	If you try and build a *.DLL while the library is active in CAPOW, you
will get an error message at link time, This will also happen if you had
a CAPOW session end with a crash or other unclean shut-down during the
current Windows session, because then Windows thinks the DLL is still in use,
and the compiler will say:
"The DLL you are linking is in use by another application.  The link will be
 aborted." To get around this you can either rename the DLL target
or you can restart Windows.
 */

//#define CARULE_01		//Call this WAVRAW??.DLL
//#define CARULE_01_A
//#define CARULE_01_B
//#define CARULE_02
//#define CARULE_03		//Call this OSCCHA??.DLL
//#define CARULE_04
//#define CARULE_05		//Call this SHELL??.DLL
#define CARULE_06		//Call this SHLWAV??.DLL

#ifdef CARULE_01
/* This is a very simple wave rule which is right on the edge of
instability.  If you resize the simulation it is likely to split
into two curves.  We don't actually need to be computing the
NEW_C_V here, but only do this in case you want to display it. */
DllExport USERRULE(CA* owner, int l, int c, int r)
{
	NEW_C_I = -PAST_C_I + (L_I + R_I);
	CLAMP(NEW_C_I, -MAX_INTENSITY, MAX_INTENSITY);
	NEW_C_V = (NEW_C_I - C_I)/DT;
}
#endif //CARULE_01

#ifdef CARULE_01_A
/* The correct way to do the wave simulation is
  	NEW_C_I = -PAST_C_I + 2.0*C_I +
		((DT*DT)/(DX*DX))*(L_I - 2.0*C_I + R_I);
Writing LAMBDA for (DT^2/DX^2), we get
NewC = -PastC + 2*(1 - LAMBDA)C + LAMBDA*(L + R)
Note that if PastC, L,C,R are all the same size S, then the expression is
-S + 2*(1-LAMBDA)*S + 2*LAMBDA*S, or -S + 2*S, or S.  The LAMBDA serves
as a weight.  We can think of it as (1-LAMBDA)(C+C) + LAMBDA(L+R).
If I take LAMBDA bigger than 1, then we get bogus values.  That's why
DT must be less than DX.  If I take, say, LAMBDA to be 1/2 I'd get
-PAST_C_I + C_I + 0.5*(L_I + R_I);  [CARULE1A]
If I were to take LAMBDA to be 2/3, I'd get
-PAST_C_I + (2.0/3.0)*C_I + (2.0/3.0)(L_I + R_I); or
-PAST_C_I + (2.0/3.0)*(L_I + C_I + R_I); [CARULE1B]
Either one of these should give a nice stable simulation.
We'll compile these as CARULE1A and CARULE1B.
It turns out that version A is quite stable, while version B is quite
unstable.
*/
DllExport USERRULE(CA* owner, int l, int c, int r)
{
	NEW_C_I = -PAST_C_I + C_I + 0.5*(L_I + R_I);
	CLAMP(NEW_C_I, -MAX_INTENSITY, MAX_INTENSITY);
	NEW_C_V = (NEW_C_I - C_I)/DT;
}
#endif //CARULE_01_A

#ifdef CARULE_01_B
/* See the commment for CARULE_O1_A
*/
DllExport USERRULE(CA* owner, int l, int c, int r)
{
	NEW_C_I = -PAST_C_I + (2.0/3.0)*(L_I + C_I + R_I);
	CLAMP(NEW_C_I, -MAX_INTENSITY, MAX_INTENSITY);
	NEW_C_V = (NEW_C_I - C_I)/DT;
}
#endif //CARULE_01_B

#ifdef CARULE_02
/* This is the same as CARULE_01 with WRAP instead of CLAMP.
It is cool with random seed.  Cool also if you put a wave
in it and then reduce max intensity with the Analog menu.  It crashes
with floating point error when you press Smooth on the World menu? */
DllExport USERRULE(CA* owner, int l, int c, int r)
{
	NEW_C_I = -PAST_C_I + (L_I + R_I);
	WRAP(NEW_C_I, -MAX_INTENSITY, MAX_INTENSITY);
	NEW_C_V = (NEW_C_I - C_I)/DT;
}
#endif //CARULE_02

#ifdef CARULE_03
/* One lack in our oscillator rule is that it never seems to give chaotic
oscillations.  Theoreticlaly you in fact need a nonlinearity in the rule.
Putting in a sine ought to work, but so far it doesn't seem to -- you can
check this with Point View */

DllExport USERRULE(CA* owner, int l, int c, int r)
{
	NEW_C_V =  C_V + (DT/MASS)*(
		- FRICTION * C_V
		- SPRING * sin(C_I) //Putting in sin here makes tnonlinear and chaotic.
      + AMPLITUDE * cos(PHASE + FREQUENCY * TIME));
	NEW_C_I = C_I + DT * NEW_C_V;
	CLAMP(NEW_C_V, -MAX_VELOCITY, MAX_VELOCITY);
	CLAMP(NEW_C_I, -MAX_INTENSITY, MAX_INTENSITY);
}
#endif //CARULE_03

#ifdef CARULE_04
//First kind of shell
DllExport USERRULE(CA* owner, int l, int c, int r)
{
	NEW_ACTIVATOR = ACTIVATOR_AVG +
		+ DT * (CELL_STRENGTH * ( ACTIVATOR_PRODUCTION)
		- INHIBITOR
		- ACTIVATOR*DECAY_RATE_ACTIVATOR);
	NEW_INHIBITOR = INHIBITOR_AVG
		+ DT * (INHIBITOR_PRODUCTION
		- ACTIVATOR
		- INHIBITOR*DECAY_RATE_INHIBITOR);
	CLAMP(NEW_ACTIVATOR, 0.0, MAX_ACTIVATOR);
	CLAMP(NEW_INHIBITOR, 0.0, MAX_INHIBITOR);
}
#endif //CARULE_04

#ifdef CARULE_05
//Second kind of shell, closer to Meinhardt book
DllExport USERRULE(CA* owner, int l, int c, int r)
{
	NEW_ACTIVATOR = ACTIVATOR_AVG
		+ DT * (CELL_STRENGTH *
			(ACTIVATOR * ACTIVATOR / (0.1 + fabs(INHIBITOR))
				+ ACTIVATOR_PRODUCTION)
		- ACTIVATOR*DECAY_RATE_ACTIVATOR);
	NEW_INHIBITOR = INHIBITOR_AVG
		+ DT * (CELL_STRENGTH * ACTIVATOR * ACTIVATOR
		+ INHIBITOR_PRODUCTION
		- INHIBITOR*DECAY_RATE_INHIBITOR);
	CLAMP(NEW_ACTIVATOR, 0.0, MAX_ACTIVATOR);
	CLAMP(NEW_INHIBITOR, 0.0, MAX_INHIBITOR);
}
#endif //CARULE_05

#ifdef CARULE_06
/* First kind of shell wave.  We say why not do wave instead of
diffusion, just to see if its gnarly.  It is indeed totally gnarly,
and fairly stable with a small dt. */
DllExport USERRULE(CA* owner, int l, int c, int r)
{
	NEW_ACTIVATOR = NEW_WAVE_ACTIVATOR  +
		+ DT *
		(CELL_STRENGTH * ( ACTIVATOR_PRODUCTION)
		- INHIBITOR
		- ACTIVATOR*DECAY_RATE_ACTIVATOR);
	NEW_INHIBITOR = NEW_WAVE_INHIBITOR
		+ DT * (INHIBITOR_PRODUCTION
		- ACTIVATOR
		- INHIBITOR*DECAY_RATE_INHIBITOR);
	CLAMP(NEW_ACTIVATOR, -MAX_ACTIVATOR, MAX_ACTIVATOR);
	CLAMP(NEW_INHIBITOR, -MAX_INHIBITOR, MAX_INHIBITOR);
}
#endif //CARULE_06

